<?php

namespace Drupal\lightbox_responsive\Plugin\Field\FieldFormatter;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Field\Attribute\FieldFormatter;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\lightbox\Plugin\Field\FieldFormatter\LightboxImageFormatterTrait;
use Drupal\responsive_image\Plugin\Field\FieldFormatter\ResponsiveImageFormatter;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'lightbox_responsive' formatter.
 */
#[FieldFormatter(
  id: 'lightbox_responsive',
  label: new TranslatableMarkup('Lightbox responsive image'),
  field_types: [
    'image',
  ],
)]
class LightboxResponsiveImageFormatter extends ResponsiveImageFormatter {

  use LightboxImageFormatterTrait;

  /**
   * The file URL generator.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected $fileUrlGenerator;

  /**
   * The breakpoint manager.
   *
   * @var \Drupal\breakpoint\BreakpointManagerInterface
   */
  protected $breakpointManager;

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->fileUrlGenerator = $container->get('file_url_generator');
    $instance->breakpointManager = $container->get('breakpoint.manager');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function addStyleToSummary(array &$summary): void {
    $lightbox_style = $this->getSetting('lightbox_style');
    $responsive_image_style = $this->responsiveImageStyleStorage->load($lightbox_style);
    if ($responsive_image_style) {
      $summary[] = $this->t('Lightbox responsive image style: @style', ['@style' => $responsive_image_style->label()]);
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function addImageStyleSelect(array &$element): void {
    $responsive_image_options = [];
    $responsive_image_styles = $this->responsiveImageStyleStorage->loadMultiple();
    uasort($responsive_image_styles, '\Drupal\responsive_image\Entity\ResponsiveImageStyle::sort');
    if (!empty($responsive_image_styles)) {
      foreach ($responsive_image_styles as $machine_name => $responsive_image_style) {
        if ($responsive_image_style->hasImageStyleMappings()) {
          $responsive_image_options[$machine_name] = $responsive_image_style->label();
        }
      }
    }

    $element['lightbox_style'] = [
      '#title' => $this->t('Lightbox responsive image style'),
      '#type' => 'select',
      '#default_value' => $this->getSetting('lightbox_style'),
      '#required' => TRUE,
      '#options' => $responsive_image_options,
    ];

  }

  /**
   * Strips image multipliers (e.g., "1x", "2x") from a given srcset string.
   *
   * This method uses a regular expression to remove any descriptors indicating
   * image density multipliers that may follow the image path in a `srcset`
   * string.
   * Common multipliers include "1x", "2x", etc. This ensures compatibility
   * when processing `srcset` for image rendering purposes.
   *
   * @param string $srcset
   *   The srcset string potentially containing multiplier descriptors.
   *
   * @return string
   *   The cleaned srcset string with multipliers removed.
   */
  protected static function stripMultiplier($srcset) {
    // Use regex to remove any multiplier descriptor (e.g., "1x", "2x") after the image path.
    $pattern = '/\s+\d+(\.\d+)?x(\s|,|$)/';
    return preg_replace($pattern, '', $srcset);
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = parent::viewElements($items, $langcode);

    /** @var \Drupal\Core\Field\EntityReferenceFieldItemListInterface $items */
    if (empty($images = $this->getEntitiesToView($items, $langcode))) {
      // Early opt-out if the field is empty.
      return $elements;
    }

    $link_class = $this->getSetting('lightbox_link_class');
    $lightbox_style_setting = $this->getSetting('lightbox_style');

    $responsive_image_style =
      $this->responsiveImageStyleStorage->load($lightbox_style_setting);

    $fallback_style = $responsive_image_style->getFallbackImageStyle();
    /** @var \Drupal\image\ImageStyleInterface $lightbox_style */
    $lightbox_style = $this->imageStyleStorage->load($fallback_style);

    /** @var \Drupal\file\FileInterface[] $images */
    foreach ($images as $delta => $image) {
      $image_uri = $image->getFileUri();

      // Add cacheability metadata from the image and image style.
      $cacheability = CacheableMetadata::createFromObject($image);
      if ($lightbox_style) {
        $cacheability->addCacheableDependency(
          CacheableMetadata::createFromObject($lightbox_style));
      }

      if ($lightbox_style && $lightbox_style->supportsUri($image_uri)) {
        $lightbox_url = $lightbox_style->buildUrl($image_uri);
      }
      else {
        $lightbox_url = $this->fileUrlGenerator->generateAbsoluteString($image_uri);
      }

      $image = $elements[$delta];

      $image_item = $image['#item'];

      $variables['uri'] = $image_item->entity->getFileUri();
      $variables['width'] = $image_item->width;
      $variables['height'] = $image_item->height;

      $breakpoints = array_reverse($this->breakpointManager->getBreakpointsByGroup($responsive_image_style->getBreakpointGroup()));

      $srcset = [];
      foreach ($responsive_image_style->getKeyedImageStyleMappings() as $breakpoint_id => $multipliers) {
        if (isset($breakpoints[$breakpoint_id])) {
          $source_attributes = _responsive_image_build_source_attributes($variables, $breakpoints[$breakpoint_id], $multipliers);
          $source_attributes_array = $source_attributes->toArray();
          $srcset[] = self::stripMultiplier($source_attributes_array['srcset']) . ' ' . $source_attributes_array['width'] . 'w';
        }
      }

      $elements[$delta] = [
        '#theme' => 'lightbox_responsive_formatter',
        '#image' => $image,
        '#lightbox_url' => $lightbox_url,
        '#lightbox_srcset' => implode(', ', $srcset),
        '#attributes' => [
          'class' => [$link_class],
        ],
      ];

      $cacheability->applyTo($elements[$delta]);

    }

    return $elements;
  }

}
