<?php

namespace Drupal\lightbox\Plugin\Field\FieldFormatter;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Field\Attribute\FieldFormatter;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\image\Plugin\Field\FieldFormatter\ImageFormatter;

/**
 * Plugin implementation of the 'lightbox' formatter.
 */
#[FieldFormatter(
  id: 'lightbox_default',
  label: new TranslatableMarkup('Lightbox image'),
  field_types: [
    'image',
  ],
)]
class LightboxImageFormatter extends ImageFormatter {

  use LightboxImageFormatterTrait;

  /**
   * {@inheritdoc}
   */
  protected function addStyleToSummary(array &$summary): void {
    $lightbox_style = $this->getSetting('lightbox_style');

    $image_styles = image_style_options(FALSE);

    unset($image_styles['']);

    if (isset($image_styles[$lightbox_style])) {
      $summary[] = $this->t('URL for lightbox image style: @style', ['@style' => $image_styles[$lightbox_style]]);
    }
    else {
      $summary[] = $this->t('Original image');
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function addImageStyleSelect(array &$element): void {
    $image_styles = image_style_options(FALSE);

    $element['lightbox_style'] = [
      '#title' => $this->t('Lightbox image style'),
      '#type' => 'select',
      '#options' => $image_styles,
      '#default_value' => $this->getSetting('lightbox_style'),
      '#empty_option' => $this->t('None (original image)'),
    ];

  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = parent::viewElements($items, $langcode);

    /** @var \Drupal\Core\Field\EntityReferenceFieldItemListInterface $items */
    if (empty($images = $this->getEntitiesToView($items, $langcode))) {
      // Early opt-out if the field is empty.
      return $elements;
    }

    $link_class = $this->getSetting('lightbox_link_class');

    $lightbox_style_setting = $this->getSetting('lightbox_style');
    /** @var \Drupal\image\ImageStyleInterface $lightbox_style */
    $lightbox_style = $this->imageStyleStorage->load($lightbox_style_setting);

    /** @var \Drupal\file\FileInterface[] $images */
    foreach ($images as $delta => $image) {
      $image_uri = $image->getFileUri();

      // Add cacheability metadata from the image and image style.
      $cacheability = CacheableMetadata::createFromObject($image);
      if ($lightbox_style) {
        $cacheability->addCacheableDependency(
          CacheableMetadata::createFromObject($lightbox_style));
      }

      if ($lightbox_style && $lightbox_style->supportsUri($image_uri)) {
        $lightbox_url = $lightbox_style->buildUrl($image_uri);
      }
      else {
        $lightbox_url = $this->fileUrlGenerator->generateAbsoluteString($image_uri);
      }

      $image = $elements[$delta];

      $elements[$delta] = [
        '#theme' => 'lightbox_formatter',
        '#image' => $image,
        '#lightbox_url' => $lightbox_url,
        '#attributes' => [
          'class' => [$link_class],
        ],
      ];

      $cacheability->applyTo($elements[$delta]);

    }

    return $elements;
  }

}
